'use strict';

import React      from 'react'
import Redirect   from 'react-router/Redirect'
import style      from './style.scss'
import {
  Textfield,
  Button,
  ProgressBar
} from 'react-mdl'

export default class CarmelComponent extends React.Component {

  constructor (props) {
    super(props)

    this.state = { email: "", password: "", trying: false, success: false }
  }

  emailUpdate(event) {
    this.setState({ email: event.target.value, trying: false, success: false })
  }

  passwordUpdate(event) {
    this.setState({ password: event.target.value, trying: false, success: false  })
  }

  signin() {
    this.props.data.client.post("login", {
          email: this.state.email,
          password: this.state.password
         }).
         then(response => {
           if (response.error) {
             this.setState({ trying: false, success: false, error: response.error })
             return
           }
           localStorage.setItem('@carmel:authToken', response.data.token)
           this.setState({ trying: false, success: true })
         }).
         catch(error => this.setState({ trying: false, success: false, error: error }))
    this.setState({ trying: true, success: false, error: null })
  }

  renderSuccessfulAuth() {
    window.location.reload()
    return (<Redirect to={this.props.onLogin || '/'}/>)
  }

  renderError() {
    if (!this.state.error) {
      return (<div/>)
    }

    return (<div styleName="error"> <p> {this.state.error} </p></div>)
  }

  renderAction() {
    if (this.state.trying) {
      return (<ProgressBar indeterminate/>)
    }
    return (<Button styleName="formField" raised colored onClick={ this.signin.bind(this) }> Sign In </Button>)
  }

  render () {
    if (this.props.session.isLoggedIn) {
      return this.renderSuccessfulAuth()
    }

    if (this.state.success) {
      return this.renderSuccessfulAuth()
    }

    return (
        <div styleName="container">
          <h2> Login </h2>
          <div styleName="content">
              <Textfield
                  disabled={this.state.trying}
                  styleName="formField"
                  floatingLabel
                  onChange={this.emailUpdate.bind(this)}
                  label="email"/>
              <Textfield
                  type="password"
                  disabled={this.state.trying}
                  styleName="formField"
                  floatingLabel
                  onChange={this.passwordUpdate.bind(this)}
                  label="password"/>
          </div>
          {this.renderError()}
          {this.renderAction()}
        </div>
    )
  }
}
