import React      from 'react'
import style      from './style.scss'
import Redirect   from 'react-router/Redirect'
import CSSModules from 'react-css-modules'

import {
  Header,
  Layout,
  Drawer,
  Content,
  Navigation,
  Footer,
  FooterSection,
  FooterLinkList,
  Button
} from 'react-mdl'

class AppLayout extends React.Component {

  constructor(props) {
    super(props)

    this.state = { loggedOut: false }
  }

  handleLogout() {
    localStorage.removeItem('@carmel:authToken')
    this.setState({ loggedOut: true })
  }

  get actionMenu () {
    return this.props.carmel.menu.map(menuItem => {
        if (!menuItem.action) {
          return
        }

        if (this.props.carmel.isLoggedIn) {
          return (<Button key={9999} raised ripple accent style={{marginLeft: "1rem"}}
                    onClick={this.handleLogout.bind(this)}>
            Logout
          </Button>)
        }

        return (<Button href={menuItem.link} key={menuItem.id} raised ripple accent style={{marginLeft: "1rem"}}>
          { menuItem.title }
        </Button>)
      })
  }

  get menu () {
    return this.props.carmel.menu.map(menuItem => {
      if (this.props.carmel.isLoggedIn) {
        if (menuItem.private) {
          return (<a key={menuItem.id} href={menuItem.link}> { menuItem.title } </a>)
        }
        return
      }

      if (menuItem.private || menuItem.action) {
        return
      }

      return (<a key={menuItem.id} href={menuItem.link}> { menuItem.title } </a>)
    })
  }

  renderHeader() {
    if (this.props.carmel.page.login) {
      return (<div></div>)
    }

    return (<Header title={this.props.carmel.info.title} style={{color: 'white'}}>
      <Navigation styleName="menu">
        { this.menu }
      </Navigation>
      { this.actionMenu}
    </Header>)
  }

  renderDrawer() {
    if (this.props.carmel.page.login) {
      return (<div></div>)
    }

    return (<Drawer title={this.props.carmel.info.title}>
      <Navigation>
        { this.menu }
      </Navigation>
    </Drawer>)
  }

  get logoutPagePath() {
    return ((this.props.carmel.loginPage && this.props.carmel.loginPage.path) ?
             this.props.carmel.loginPage.path : "/login")
  }

  render () {
    if (this.state.loggedOut) {
      return (<Redirect to={this.logoutPagePath}/>)
    }

    return (<Layout fixedHeader>
            { this.renderHeader() }
            { this.renderDrawer() }
            <Content>
              <div styleName="container">
                { this.props.children}
              </div>
            </Content>
      </Layout>)
  }
}

export default CSSModules(AppLayout, style);
